;; Generated package description from smart-yank.el  -*- no-byte-compile: t -*-
(define-package "smart-yank" "0.1.1.0.20221221.82231" "A different approach of yank pointer handling" '((emacs "24")) :commit "5494fb8f1cfb89561cdebdd34cd7f899203cbc52" :url "https://elpa.gnu.org/packages/smart-yank.html" :authors '(("Michael Heerdegen" . "michael_heerdegen@web.de")) :maintainer '("Michael Heerdegen" . "michael_heerdegen@web.de") :keywords '("convenience"))
